@@ -45,12 +45,14 @@
 #include "photos-export-dialog.h"
 #include "photos-export-notification.h"
 #include "photos-filterable.h"
+#include "photos-gegl.h"
 #include "photos-glib.h"
 #include "photos-item-manager.h"
 #include "photos-main-window.h"
 #include "photos-properties-dialog.h"
 #include "photos-query.h"
 #include "photos-resources.h"
+#include "photos-resources-gegl.h"
 #include "photos-search-context.h"
 #include "photos-search-controller.h"
 #include "photos-search-match.h"
@@ -76,6 +78,7 @@ struct _PhotosApplication
   GList *miners;
   GList *miners_running;
   GResource *resource;
+  GResource *resource_gegl;
   GSettings *bg_settings;
   GSettings *ss_settings;
   GSimpleAction *blacks_exposure_action;
@@ -117,6 +120,7 @@ struct _PhotosApplication
   TrackerExtractPriority *extract_priority;
   gboolean main_window_deleted;
   guint create_miners_count;
+  guint init_fishes_id;
   guint use_count;
   guint32 activation_timestamp;
   gulong source_added_id;
@@ -553,6 +557,17 @@ photos_application_create_miners (PhotosApplication *self)
 }
 
 
+static gboolean
+photos_application_gegl_init_fishes_idle (gpointer user_data)
+{
+  PhotosApplication *self = PHOTOS_APPLICATION (user_data);
+
+  self->init_fishes_id = 0;
+  photos_gegl_init_fishes ();
+  return G_SOURCE_REMOVE;
+}
+
+
 static gboolean
 photos_application_sanity_check_gegl (PhotosApplication *self)
 {
@@ -653,6 +668,9 @@ photos_application_create_window (PhotosApplication *self)
   self->main_window_deleted = FALSE;
   self->factory = photos_thumbnail_factory_dup_singleton (NULL, NULL);
 
+  if (self->init_fishes_id == 0)
+    self->init_fishes_id = g_idle_add (photos_application_gegl_init_fishes_idle, self);
+
   g_application_hold (G_APPLICATION (self));
   tracker_extract_priority_proxy_new_for_bus (G_BUS_TYPE_SESSION,
                                               G_DBUS_PROXY_FLAGS_NONE,
@@ -1609,6 +1627,12 @@ photos_application_shutdown (GApplication *application)
   refresh_miner_ids_size = g_hash_table_size (self->refresh_miner_ids);
   g_assert (refresh_miner_ids_size == 0);
 
+  if (self->init_fishes_id != 0)
+    {
+      g_source_remove (self->init_fishes_id);
+      self->init_fishes_id = 0;
+    }
+
   g_clear_pointer (&self->refresh_miner_ids, (GDestroyNotify) g_hash_table_unref);
 
   G_APPLICATION_CLASS (photos_application_parent_class)->shutdown (application);
@@ -1669,6 +1693,9 @@ photos_application_startup (GApplication *application)
   self->resource = photos_get_resource ();
   g_resources_register (self->resource);
 
+  self->resource_gegl = photos_gegl_get_resource ();
+  g_resources_register (self->resource_gegl);
+
   icon_theme = gtk_icon_theme_get_default ();
   gtk_icon_theme_add_resource_path (icon_theme, "/org/gnome/Photos/icons");
 
@@ -1896,6 +1923,12 @@ photos_application_dispose (GObject *object)
       self->resource = NULL;
     }
 
+  if (self->resource_gegl != NULL)
+    {
+      g_resources_unregister (self->resource_gegl);
+      self->resource_gegl = NULL;
+    }
+
   g_clear_object (&self->create_window_cancellable);
   g_clear_object (&self->bg_settings);
   g_clear_object (&self->ss_settings);
